-------------------------------------------------------------------------------------
-- | Namespace Definitions |---------------------------------------------------------
-------------------------------------------------------------------------------------

-- Retrieve the addon name and the namespace table.
local addon, npcg = ...

local functua 	= npcg.functua
local stringLib = npcg.functua.string
local util 			= npcg.functua.util
local version   = "1.6.2"

local system 	= npcg.system

npcGenerator = { }
NPCG            = NPCG            or { }
NPCG.Events     = NPCG.Events     or { }

local characterTemplates = { }
system.characterTemplates = characterTemplates

-- Todo: Set local
function npcg.system.getCurrentCharacterTemplate()
  return characterTemplates[npcGenerator_NPC_Frame.Tabs.currentTab]
end

local GetCurrentCharacterTemplate = npcg.system.getCurrentCharacterTemplate

function GetCharacterTemplateInTab(tabNumber)
  return characterTemplates[tabNumber]
end

function RemakeGenerator()
  npcg.generators.race.initialize()
  npcg.generators.tribe.initialize()
  npcg.generators.class.initialize()
end

--[[ Save NPC
function SaveNPC(npc, saveSlot)
	print("SaveNPC called!")
	
  local i = npcGenerator_NPC_Frame.Tabs.CurrentTab
  if not stringLib.nilOrWhitespace(npc.name) then
    local npcName = npc.name
    savedNpcTemplates[saveSlot] = CharacterTemplate.clone(npc)
    npcGenerator_NPC_Frame.Tabs.ContextFrame:Hide()
  else
    npcGenerator_NPC_Frame.HelperMessageFrame:AddMessage("The npcs needs a name to be saved.", 1.00, 0.05,0.05, 1.0)
  end
  --print "The npc is not well defined, some vital traits are lacking"
end
]]

--[[ Load NPC
-- Is Called!
function LoadNPC(npc)
	print("LoadNPC called!")
	
  -- Current open tab.
  local frame = npcGenerator_NPC_Frame
  local n     = frame.Tabs.currentTab

  -- The npc parameter is a clone, not a reference, so we can simply link to that, instead of making a new clone.
  characterTemplates[n] = npc
  printToFrame(GetCurrentCharacterTemplate())
end
]]

--[[
function DeleteNPC(index)
	print("DeleteNPC called!")
  table.remove(savedNpcTemplates, index)
end
]]

function npcGenerator.OnLoad()
  print "Hello, this is your pocket gnome kindly asking you to report npcGenerator issues to our issue tracker at https://wow.curseforge.com/projects/npcgenerator/issues"
  table.insert(characterTemplates, CharacterTemplate:new())
  
  local frame = npcGenerator_NPC_Frame
  
  -- Set up the Labels correctly. REDO THIS LATER! THIS SHOULD BE IN UI
  frame.Title.Label:SetText(       "npcGenerator " .. version)
	
  -- Hook onto Tab Changes.
	frame.Tabs.OnTabAdded:AddListener(
		function(event, self, ...)
			table.insert(characterTemplates, CharacterTemplate:new())
			npcg.io.WriteToFrame(GetCurrentCharacterTemplate())
		end)
	frame.Tabs.OnTabRemoved:AddListener(
		function(event, self, removedTabIndex, ...)
			table.remove(characterTemplates, removedTabIndex)
		end)
	frame.Tabs.OnTabChanged:AddListener(
		function(event, self, removedTabIndex, ...)
		npcg.io.WriteToFrame(GetCurrentCharacterTemplate())
	end)
	
	-- Should ContextFrame be part of Tabs?
  frame.Tabs.ContextFrame:SetScript("OnShow", function(self)
    local ContextFrameTitle = ""
    if stringLib.nilOrWhitespace(characterTemplates[self.GetTabIndex()]:getName()) then
      ContextFrameTitle = "Unamed npc " .. self.GetTabIndex()
      frame.Tabs.ContextFrame.SaveButton:Disable()
      frame.Tabs.ContextFrame.SaveButton.Text:SetTextColor("0.5", "0.5", "0.5", "1.0")
    elseif frame.Tabs.ContextFrame.GetTabIndex() ~= frame.Tabs.currentTab then
      ContextFrameTitle = characterTemplates[self.GetTabIndex()]:getName()
      frame.Tabs.ContextFrame.SaveButton:Disable()
      frame.Tabs.ContextFrame.SaveButton.Text:SetTextColor("0.5", "0.5", "0.5", "1.0")
    else
      ContextFrameTitle = characterTemplates[self.GetTabIndex()]:getName()
      frame.Tabs.ContextFrame.SaveButton:Enable()
      frame.Tabs.ContextFrame.SaveButton.Text:SetTextColor("1.0", "0.82", "0.0", "1.0")
    end
    
    self["Label"]:SetText(stringLib.ucWords(ContextFrameTitle))
  end)
  
  frame.Tabs.ContextFrame.LoadButton:SetScript("OnClick", function()
    npcGenerator_Load_Frame:Show()
    frame.Tabs.ContextFrame:Hide()
  end)
  frame.BottomBar.LoadButton:SetScript("OnClick", function() npcGenerator_Load_Frame:Show() end)
  
  local onSaveClicked = function()
    if not stringLib.nilOrWhitespace(GetCurrentCharacterTemplate().name) then
      npcGenerator_Save_Frame:Show()
    else
      error("The npc needs a name to be saved!")
    end
    npcGenerator_NPC_Frame.Tabs.ContextFrame:Hide()
  end
  
  frame.Tabs.ContextFrame.SaveButton:SetScript("OnClick", onSaveClicked)
  frame.BottomBar.SaveButton:SetScript(        "OnClick", onSaveClicked)
  
	-- In Validate NPC Frame
	-- DEPRICATED
  local onTribeSet = function()
    local tribeContext = npcg.data.tribe.getTribeContext(GetCurrentCharacterTemplate():getRace())
    local hasTribe = not stringLib.nilOrWhitespace(tribeContext)
    
    if not hasTribe then
			-- Disable()
      frame.Tribe.Label:SetTextColor(0.5, 0.5, 0.5)
      frame.Tribe.Label:SetText("Tribe")
      frame.Tribe.EditBox:SetTextColor(0.5, 0.5, 0.5)
      frame.Tribe.EditBox:SetText("")
      frame.Tribe.EditBox:Disable()
      frame.Tribe.RollButton:Disable()
    else
			-- Enable()
      frame.Tribe.EditBox:SetTextColor(1.0, 1.0, 1.0)
      frame.Tribe.EditBox:Enable()
      frame.Tribe.EditBox:SetText("")
      frame.Tribe.Label:SetTextColor(1.0, 1.0, 1.0)
      frame.Tribe.Label:SetText(tribeContext)
      frame.Tribe.RollButton:Enable()
    end
    
    
  end
  
  local createDepencencyOn = function(item, listOfDependancies)
    local allRequirementsMet = function() return
      functua.all(function(req) return
        not stringLib.nilOrWhitespace(frame[req].EditBox:GetText()) end,
        listOfDependancies)
    end
    
    local enable = function(uiElement)
			-- Define this in the items.
      if uiElement ~= "Motivation" then -- This is hard coded. Instead, use traitDefinition:hasMultipleFields
        frame[uiElement].RollButton:Enable()
      else
        --frame.Motivations.Label:SetTextColor( 1.0, 1.0, 1.0) -- Why is this..?
        functua.iterate(function(i) frame.Motivations["RollButton" .. i]:Enable() end, 1, 3)
      end
    end
    
    local disable = function(uiElement)
			-- Define this in the items.
      if uiElement ~= "Motivation" then -- This is hard coded. Instead, use traitDefinition:hasMultipleFields
        frame[uiElement].RollButton:Disable()
      else
        functua.iterate(function(i) frame.Motivations["RollButton" .. i]:Disable() end, 1, 3)
      end
    end
    
    functua.forEach(
			function(dep)
				frame[dep].EditBox:HookScript("OnTextChanged",
					function()
						if allRequirementsMet() then enable(item) else disable(item) end
        end)
			end,
      listOfDependancies)
  end
  
	-- Store this in TraitDefinitions
  createDepencencyOn("Class",   		{ "Race", "Age", "Gender", })
	createDepencencyOn("Tribe", 			{ "Race" })
	createDepencencyOn("Faith",   		{ "Alignment" })
  createDepencencyOn("Feature", 		{ "Race", "Class", "Gender", "Age" })
  createDepencencyOn("Name",    		{ "Race" })
  createDepencencyOn("Motivation", 	{ "Race", "Class", "Age", "Faith", "Gender", "Alignment" })
  
	-- This is UI stuff.
  frame.Race.EditBox:SetScript("OnEditFocusLost", onTribeSet)
  frame.Race.EditBox:SetScript("OnTextSet",       onTribeSet)

  onTribeSet()
end

-- Redo modules
function NPCG.Events.OnAddonLoad()
	-- Extract to LoadOptions function. Add to options.lua.
  if savedNpcTemplates == nil then
    savedNpcTemplates = { }
  end
  if NPCG_Options == nil then
    NPCG_Options = { 
      debug="false", 
      safeMode="false",
      factionSettings={
        matching = true,
        context  = true,
      },
      locationSettings={
        zones    = true,
        subZones = true,
        mock     = false,
        mockZone = "",
        mockSub  = false,
        mockSubZone = "",
      },
      friendSettings={
        friends  = true,
				changed	 = 0.1,
      },
			interface={
				minimapPosition = 45, -- Position in deg, ccw.
				minimapPositionX = 0, -- Position for unlocked minimap button. 
				minimapPositionY = 0, -- Position for unlocked minimap button. 
				minimapLockedToMap = true,
				displayMinimapButton = true,
				enableTooltips = true,
			},
    }  
  else
    NPCG_Options.debug         = NPCG_Options.debug         or "false"
    NPCG_Options.factionSettings  = NPCG_Options.factionSettings  or { }
    if NPCG_Options.factionSettings.matching  == nil then NPCG_Options.factionSettings.matching  = true end
    if NPCG_Options.factionSettings.context   == nil then NPCG_Options.factionSettings.context   = true end
    NPCG_Options.locationSettings = NPCG_Options.locationSettings or { }
    if NPCG_Options.locationSettings.zones    == nil then NPCG_Options.locationSettings.zones    = true end
    if NPCG_Options.locationSettings.subZones == nil then NPCG_Options.locationSettings.subZones = true end
    if NPCG_Options.locationSettings.mock     == nil then NPCG_Options.locationSettings.mock     = true end
    NPCG_Options.locationSettings.mockZone    = NPCG_Options.locationSettings.mockZone or ""
    if NPCG_Options.locationSettings.mockSub  == nil then NPCG_Options.locationSettings.mockSub  = true end
    NPCG_Options.locationSettings.mockSubZone = NPCG_Options.locationSettings.mockSubZone or ""
    NPCG_Options.friendSettings   = NPCG_Options.friendSettings   or { }
    if NPCG_Options.friendSettings.friends    == nil then NPCG_Options.friendSettings.friends    = false end
		NPCG_Options.friendSettings.change        = NPCG_Options.friendSettings.chance or 0.1
    if NPCG_Options.raceSettings  ~= nil then
      NPCG_Options.raceSettings = nil
    end
    if NPCG_Options.tribeSettings  ~= nil then
      NPCG_Options.tribeSettings = nil
    end
		if NPCG_Options.interface == nil then
			NPCG_Options.interface = { }
		end
		NPCG_Options.interface.minimapPosition 			= NPCG_Options.interface.minimapPosition or 45
		if NPCG_Options.interface.displayMinimapButton == nil then
			NPCG_Options.interface.displayMinimapButton = true
		end
		if NPCG_Options.interface.enableTooltips == nil then
			NPCG_Options.interface.enableTooltips = true
		end
    NPCG_Options.safeMode      = NPCG_Options.safeMode      or "false"
  end

	-- Rename Function to Options Frames.
  NPCG.LoadOptions()

	-- RemakeGenerator should be in Initialize.
  RemakeGenerator()
  if not (NPCG_Options.locationSettings.zones and NPCG_Options.locationSettings.mock) then
    npcGenerator_settings_Frame.Panel.MockZoneEditBox:Disable()
    npcGenerator_settings_Frame.Panel.MockZoneEditBox:SetTextColor(0.5, 0.5, 0.5)
  end
  
  if not (NPCG_Options.locationSettings.zones and NPCG_Options.locationSettings.subZones and NPCG_Options.locationSettings.mockSub) then
    npcGenerator_settings_Frame.Panel.MockSubZoneEditBox:Disable()
    npcGenerator_settings_Frame.Panel.MockSubZoneEditBox:SetTextColor(0.5, 0.5, 0.5)
  end
	
	-- Reposition the Minimap Button
	npcGenerator_Minimap_Button_OnAddonLoaded()
  
end

-- Redo modules
function NPCG.Events.OnAddonUnload()
  
end

local eventFrame = CreateFrame("FRAME")

eventFrame:RegisterEvent("ADDON_LOADED")
eventFrame:RegisterEvent("PLAYER_LOGOUT")

function eventFrame.OnEvent(unusedArg, event, name)
  if name == "npcGenerator" then
    if     event == "ADDON_LOADED"  then NPCG.Events.OnAddonLoad()
    elseif event == "PLAYER_LOGOUT" then NPCG.Events.OnAddonUnload() end
  end
end
eventFrame:SetScript("OnEvent", eventFrame.OnEvent)

-- End of File.